/* Display TIAN-MA am I2C fr die LMK-Radio-Frontplatte  */

/*
Pollin Nr. 121581 "Tian_Ma A2C00096100"
Wir haben hier 20 Alpha-Zeichen (0..19) und einige Icons
*/

#include "StdTypes.h"  // wegen byte, bool, dword usw.
#include "LPC111X.h"   // wegen RESET_high() und .._low()
#include "systick.h"   // wegen Wartezeiten beim Setup
#include "i2c.h"       // fr die I2C-Ansteuerung
#include "tianma.h"

#define  LcdAdr  0x3A


/********* Direktbedienung des Reset-Pins am Display ************/
// Display-Reset ansteuern
void RESET_high (void)
{ GPIO1_DATA |= 1<<9; }

void RESET_low (void)
{ GPIO1_DATA &= ~(1<<9); }

/****************************************************************/

char LCDBUF[cbuflen];
byte LCDIdx;

byte ICONBUF[8];

// bersetzung Stringposition --> LCDBUF-Position
const byte DD_Idx[32] =
{ 0x1A,0x1B,0x1C,0x1E,0x1F,
  0x00,0x01,0x02,0x03,0x04,
  0x05,0x06,0x07,0x08,0x09,
  0x0A,0x0B,0x0C,0x0E,0x0F,
  0x0D,0x10,0x11,0x12,0x13,
  0x14,0x15,0x16,0x17,0x18,
  0x19,0x1D
};

// Die Sonder-Zeichen 0..3 sind nicht benutzbar,
// da sie von den Icons belegt sind

/* Icons:
 1 = Folder,    2 = RND,   3 = NEWS,  4 = LOC  5 = PTY
 6 = Rand um 7, 7 = TP,    8 = TA,    9 = -L  10 = Ergnzung 9 zu -D
 damit 8+9+10 =  TA oder TA-L oder TA-D
11 = AST,       12 = FM,  13 = Ergnzung 12 zu AM, 14..20 = A..G linke Nummer
 damit  FM 0..9  oder AM 0..9
 oder FM
      AM
      AML   (SSB: LSB)
      AMU   (SSB: USB)
21 = P
22..28 A..G rechte Nummer
 damit P 0..9
29 = Stereo-Symbol
30..36 = Schrgstrich zw. drittletztem+zweitletztem Textzeichen,
         (nicht benutzen, macht Probleme)
*/

// Icon-belegung (CGRAM Adressen ab 8, also Userchar 1)
const byte Icons[29] =
{ (2<<5)+(1<<4), // Folder
  (2<<5)+(1<<3), // RND
  (2<<5)+(1<<2), // NEWS
  (2<<5)+(1<<1), // LOC
  (2<<5)+(1<<0), // PTY
  (3<<5)+(1<<3), // frame around icon 7
  (3<<5)+(1<<4), // TP
  (3<<5)+(1<<2), // TA
  (3<<5)+(1<<1), // -L
  (3<<5)+(1<<0), // -D (together with icon 9)
  (4<<5)+(1<<4), // AST
  (4<<5)+(1<<3), // FM
  (4<<5)+(1<<2), // AM (together with icon 12)
  (6<<5)+(1<<0), // left seven segment A
  (6<<5)+(1<<1), // left seven segment B
  (6<<5)+(1<<4), // left seven segment C
  (4<<5)+(1<<1), // left seven segment D
  (4<<5)+(1<<0), // left seven segment E
  (6<<5)+(1<<2), // left seven segment F
  (6<<5)+(1<<3), // left seven segment G
  (7<<5)+(1<<4), // P
  (1<<5)+(1<<1), // right seven segment A
  (1<<5)+(1<<2), // right seven segment B
  (7<<5)+(1<<1), // right seven segment C
  (7<<5)+(1<<3), // right seven segment D
  (7<<5)+(1<<2), // right seven segment E
  (1<<5)+(1<<3), // right seven segment F
  (7<<5)+(1<<0), // right seven segment G
  (1<<5)+(1<<0)  // stereo symbol
};

// Belegung der Zeichen 4..11 als Pegelanzeige.
#define cgnum  (8 * 8)
const byte CG_Var[cgnum] =
{ /*  0 */ 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x10, 0,
  /*  1 */ 0x10, 0x00, 0x10, 0x00, 0x10, 0x00, 0x17, 0,
  /*  2 */ 0x10, 0x00, 0x10, 0x00, 0x10, 0x07, 0x17, 0,
  /*  3 */ 0x10, 0x00, 0x10, 0x00, 0x17, 0x07, 0x17, 0,
  /*  4 */ 0x10, 0x00, 0x10, 0x07, 0x17, 0x07, 0x17, 0,
  /*  5 */ 0x10, 0x00, 0x17, 0x07, 0x17, 0x07, 0x17, 0,
  /*  6 */ 0x10, 0x07, 0x17, 0x07, 0x17, 0x07, 0x17, 0,
  /*  7 */ 0x17, 0x07, 0x17, 0x07, 0x17, 0x07, 0x17, 0
};



const byte LcdSetup[11] =
{ 0x00, // Control Byte (CO=0/RS=0)
  0x20, // Function Set (Base + Standard, 1 Line, 1:18 Multiplex)
  0x06, // Entry Mode   (Addr Increment, No Shift)
  0x0E, // Display Ctl  (Disp On, Cursor On), Cursor nicht sichtbar, da Zeile 8
  0x21, // Function Set (Base + Extended, 1 Line, 1:18 Multiplex)
  0x04, // Disp_conf    (Column: Left to Right, Row: Top to Bottom)
  0x42, // HV_gen       (3 voltage multiplier)
  0x08, // Icon_ctl     (Char Mode, No Icon Blink, Direct Mode Off)
  0xB7, // VLCD_set: VA: 0x3A=6.6V, 0x37=6.3V,
  0xC0, // VLCD_set     (VB=0x00) Disabled
  0x20  // Function Set (Base + Standard, 1 Line, 1:18
};


/************** LCDBUF ausgeben **************************/
bool LCD_BufOut (void)
{ int i;
  bool erg;

  erg = Do_I2C_Start(LcdAdr, I2CWRITE);
  if (!erg) goto raus;

  erg = Do_I2C_Write(0x80);  // CO=1, RS=0 -> Control schreiben
  if (!erg) goto raus;

  erg = Do_I2C_Write(2);     // Home
  if (!erg) goto raus;

  erg = Do_I2C_Write(0x80);  // CO=1, RS=0 -> Control schreiben
  if (!erg) goto raus;

  erg = Do_I2C_Write(0x80);  // set DDRAM Adr
  if (!erg) goto raus;

  erg = Do_I2C_Write(0x40);  // CO=0, RS=1 -> Daten schreiben
  if (!erg) goto raus;

  i = 0;
  while (i<cbuflen)
  { erg = Do_I2C_Write(LCDBUF[i]);
    if (!erg) goto raus;
    ++i;
  }
  raus:
  Do_I2C_Stop();
  return erg;
}

/************** ab der aktuellen Schreibposition LCD lschen *******/
void LCD_ClrEol (void)
{ int i, j;

  i = LCDIdx;
  while (i < cbuflen)
  { j = DD_Idx[i];
    LCDBUF[j] = ' ';
    ++i;
  }
  LCD_BufOut();
}

/************** LCD lschen *************************************/
void LCD_Clear (void)
{ LCDIdx = 0;
  LCD_ClrEol();
}

/*************** Zeichen auf dem Display ausgeben ****************/
char LCD_CharOut(char c)
{ int j;

  if (LCDIdx >= cbuflen) return false;
  j = DD_Idx[LCDIdx];
  ++LCDIdx;
  LCDBUF[j] = c;
  return c;
}

/************ Zeichen auf dem Display an Stelle 'pos' ausgeben *****/
char LCD_CharAt  (char c, byte pos, bool mitAktu)
{ int j;

  if (pos >= cbuflen) return false;
  j = DD_Idx[pos];
  LCDBUF[j] = c;
  if (mitAktu) LCD_BufOut();
  LCDIdx = pos+1;
  return c;
}

/************* Schreibposition setzen **********************/
void LCD_SetPos  (int Position)
{ if (Position < 0)  Position = 0;
  if (Position > 19) Position = 19;
  LCDIdx = Position;
}

/************* Icons setzen und lschen. Zhlweise: ab 1 *************/
// bei IconNummer=0 werden alle gelscht
bool SetIcon (int IconNummer, bool visible)
{ int  adr;
  byte code;
  bool erg;

  if (IconNummer < 1 )
  { adr = 0;
    while (adr<8) ICONBUF[adr++] = 0;
    goto IconAnzeigen;
  }
  if (IconNummer > 36) return false;
  adr = Icons[IconNummer-1];
  code = adr & 0x1F;
  adr = adr>>5;

  if (visible)
    ICONBUF[adr] |= code;
  else
    ICONBUF[adr] &= ~code;

  IconAnzeigen:
  erg = Do_I2C_Start(LcdAdr, I2CWRITE);
  if (!erg) goto raus;

  erg = Do_I2C_Write(0x80);  // CO=1, RS=0 -> Control schreiben
  if (!erg) goto raus;

  erg = Do_I2C_Write(0x80);  // set DDRAM Adr auf 0 (lscht Bit 6)
  if (!erg) goto raus;

  erg = Do_I2C_Write(0x80);  // CO=1, RS=0 -> Control schreiben
  if (!erg) goto raus;

  erg = Do_I2C_Write(0x48);  // set CGRAM Adr auf 8, also Anfang Zeichen 1
  if (!erg) goto raus;

  erg = Do_I2C_Write(0x40);  // CO=0, RS=1 -> Daten schreiben
  if (!erg) goto raus;

  adr = 0;
  while (adr<8)
  { Do_I2C_Write(ICONBUF[adr]);

    if (!erg) goto raus;
    ++adr;
  }
  raus:
  Do_I2C_Stop();
  return erg;
}


/************ Display initialisieren *************/
bool LCD_Init (void)
{ int i;
  char c;
  bool erg;

  // Reset applizieren
  RESET_high();
  Warte_ms(3);
  RESET_low();
  Warte_ms(20);

  // Setup der Register
  Do_I2C_Stop();
  erg = Do_I2C_Start(LcdAdr, I2CWRITE);
  if (!erg) goto raus;

  i = 0;
  while (i<11)
  { erg = Do_I2C_Write(LcdSetup[i]);
    if(!erg) return false;
    ++i;
  }
  Do_I2C_Stop();

  erg = Do_I2C_Start(LcdAdr, I2CWRITE);
  if (!erg) goto raus;

  erg = Do_I2C_Write(0x80);  // CO=1, RS=0 -> Control schreiben
  if (!erg) goto raus;

  erg = Do_I2C_Write(0x80);  // set DDRAM Adr auf 0 (lscht Bit 6)
  if (!erg) goto raus;

  erg = Do_I2C_Write(0x80);  // CO=1, RS=0 -> Control schreiben
  if (!erg) goto raus;

  erg = Do_I2C_Write(0x40);  // set CGRAM Adr
  if (!erg) goto raus;

  erg = Do_I2C_Write(0x40);  // CO=0, RS=1 -> Daten schreiben
  if (!erg) goto raus;

  /* die Sonderzeichen und Icons initialisieren */
  /* Sonderzeichen 0..3: je 8*5bit = Icons, */
  i = 0;
  while (i<128)
  { c = 0;
    if (i<29) ICONBUF[i] = c;
    if (i>=32)  // Sonderzeichen ab Char 4
    { if ((i-32)<cgnum) c = CG_Var[i-32];
    }
    erg =Do_I2C_Write(c);
    if (!erg) goto raus;
    ++i;
  }

  raus:
  Do_I2C_Stop();

  // Display leeren
  LCD_Clear();
  return erg;
}


 /***/

